/*------------------------------------------------------------------------
    File        : postprocess.p
    Purpose     : Optional Special Post-Processing which does not fit into the main flow
    Author(s)   : Thomas Mercer-Hursh
    Created     : Mon Mar 10 15:51:23 PDT 2008
    Notes       :
  ----------------------------------------------------------------------*/
using com.cintegrity.Lib.Exception.*.
/* ***************************  Definitions  ******************************** */
                                                               /* "Parameters */
define variable lchLookupProgSourceFile as character no-undo initial "allLookupTargets.txt".  
                                                                   /* Classes */
define variable lobExceptionStack as class SimpleExceptionStack no-undo.
                                                                     /* GUIDs */
define variable lchMenuProgramGUID as character no-undo.
define variable lchMenuProgramCUGUID as character no-undo.
define variable lchLookupGUID as character no-undo.
define variable lchLookupCUGUID as character no-undo.
define variable lchProgramGUID as character no-undo.
define variable lchCompileUnitGUID as character no-undo.
define variable lchRunProgramGUID as character no-undo.
define variable lchRunMenuGUID as character no-undo.
define variable lchRunMenuCUGUID as character no-undo.
                                                            /* Work variables */
define variable lchProgramName as character no-undo.
define variable lchCompileUnitName as character no-undo.
define variable lchCompileName as character no-undo.
define variable lchDiskFile as character no-undo.
                                                                   /* Buffers */
define buffer lbfObject for Repository.t_Object.
define buffer lbfXref for Repository.t_Xref.
define buffer lbfXref2 for Repository.t_Xref.
define buffer lbfConnector for Repository.t_Connector.
                                                               /* Temp-Tables */
define temp-table ttLookupProgram no-undo
    field chProgramName as character
    index ixByName as unique primary chProgramName.
/* *****************************  Functions ********************************* */
function sfCompileName returns character ( 
    input ipchProgramName as character ) 
    in super. 
/* *************************** Initializations ****************************** */
lobExceptionStack = new SimpleExceptionStack().
lobExceptionStack:trace( "Start of Custom Postprocessing at " + string(now) ).
run spFindStereotype (
     input "oeMenuProgram",
     input "Component",
     output lchMenuProgramGUID
     ).
run spFindStereotype (
     input "oeMenuProgramCU",
     input "Package",
     output lchMenuProgramCUGUID
     ).
run spFindStereotype (
     input "oeLookup",
     input "Component",
     output lchLookupGUID
     ).
run spFindStereotype (
     input "oeLookupCU",
     input "Package",
     output lchLookupCUGUID
     ).
run spFindStereotype (
     input "oeProgram",
     input "Component",
     output lchProgramGUID
     ).
run spFindStereotype (
     input "oeCompileUnit",
     input "Package",
     output lchCompileUnitGUID
     ).              
run spFindStereotype (
     input "oeRunProgram",
     input "ControlFlow",
     output lchRunProgramGUID
     ).
run spFindStereotype (
     input "oeRunMenu",
     input "ControlFlow",
     output lchRunMenuGUID
     ).
run spFindStereotype (
     input "oeRunMenuCU",
     input "ControlFlow",
     output lchRunMenuCUGUID
     ).
                                             /* Add list of oeLookup programs */
lchDiskFile = search( lchLookupProgSourceFile ).
if lchDiskFile eq ?
then do:
  lobExceptionStack:pushException( 899, 1, "Cannot find " + lchLookupProgSourceFile + " - aborting" ).
  return.
end.
input from value( lchDiskFile ) no-echo.
repeat:
  import lchProgramName.
  if lchProgramName ne ? and lchProgramName ne ""
  then do:  
    create ttLookupProgram.
    assign
        ttLookupProgram.chProgramName = lchProgramName.
        .
    lchCompileName = sfCompileName( lchProgramName ).
    create ttLookupProgram.
    assign
        ttLookupProgram.chProgramName = lchCompileName.
        .
  end.
end.
input close.      
/* ***************************  Main Block  ********************************* */
for each lbfObject exclusive-lock:
  find lbfXref
      where lbfXref.Client = lbfObject.EA_GUID
      no-error.
  if lbfObject.Name begins "analyst/m_"
  then do:
    case lbfObject.Object_Type:
      when "Component"
        then do:
          if available ( lbfXref )
          then do:
            lbfXref.Description = replace( lbfXref.Description, lbfObject.Stereotype, "oeMenuProgram" ).
            lbfXref.Description = replace( lbfXref.Description, lchProgramGUID, lchMenuProgramGUID ).
          end.    
          lbfObject.Stereotype = "oeMenuProgram".
          for each lbfConnector
              where lbfConnector.End_Object_ID eq lbfObject.Object_ID
              :
            if lbfConnector.Stereotype eq "oeRunProgram"
            then do:
              find lbfXref2
                  where lbfXref2.Client eq lbfConnector.EA_GUID
                  no-error.
              if available ( lbfXref2 )
              then do:
                lbfXref2.Description = replace( lbfXref.Description, lbfObject.Stereotype, "oeRunMenu" ).
                lbfXref2.Description = replace( lbfXref.Description, lchRunProgramGUID, lchRunMenuGUID ).
              end.               
              lbfConnector.Stereotype = "oeRunMenu".
            end.
          end.
        end.
      when "Package"
        then do:
          if available ( lbfXref )
          then do:
            lbfXref.Description = replace( lbfXref.Description, lbfObject.Stereotype, "oeMenuProgramCU" ).
            lbfXref.Description = replace( lbfXref.Description, lchCompileUnitGUID, lchMenuProgramCUGUID ).
          end.    
          lbfObject.Stereotype = "oeMenuProgramCU".
          for each lbfConnector
              where lbfConnector.End_Object_ID eq lbfObject.Object_ID
              :
            if lbfConnector.Stereotype eq "oeRunProgram"
            then do:
              find lbfXref2
                  where lbfXref2.Client eq lbfConnector.EA_GUID
                  no-error.
              if available ( lbfXref2 )
              then do:
                lbfXref2.Description = replace( lbfXref.Description, lbfObject.Stereotype, "oeRunMenuCU" ).
                lbfXref2.Description = replace( lbfXref.Description, lchRunProgramGUID, lchRunMenuCUGUID ).
              end.               
              lbfConnector.Stereotype = "oeRunMenuCU".
            end.
          end.
        end.
      otherwise 
        do:
          lobExceptionStack:pushException( 895, 2, "Unexpected element type " + lbfObject.Object_Type + " for " + lbfObject.Name ). 
        end.
    end case.
  end.  
  else if lbfObject.Name begins "analyst/lookup"
  then do:
    if lbfObject.Stereotype eq "oeProgram"
    then do:
      if available ( lbfXref )
      then do:
        lbfXref.Description = replace( lbfXref.Description, lbfObject.Stereotype, "oeLookup" ).
        lbfXref.Description = replace( lbfXref.Description, lchProgramGUID, lchLookupGUID ).
      end.    
      lbfObject.Stereotype = "oeLookup".
    end.
    else if lbfObject.Stereotype eq "oeCompileUnit"
    then do:

      if available ( lbfXref )
      then do:
        lbfXref.Description = replace( lbfXref.Description, lbfObject.Stereotype, "oeLookupCU" ).
        lbfXref.Description = replace( lbfXref.Description, lchProgramGUID, lchLookupCUGUID ).
      end.    
      lbfObject.Stereotype = "oeLookupCU".
    end.
    else do:
      lobExceptionStack:pushException( 896, 2, "Unexpected element type " + lbfObject.Object_Type + " for " + lbfObject.Name ). 
    end.
  end.
  else do:
    find ttLookupProgram 
        where lbfObject.Name eq ttLookupProgram.chProgramName
        no-error.
    if available ( ttLookupProgram )
    then do:
      case lbfObject.Object_Type:
        when "Component"
          then do:    
            if available ( lbfXref )
            then do:
              lbfXref.Description = replace( lbfXref.Description, lbfObject.Stereotype, "oeLookup" ).
              lbfXref.Description = replace( lbfXref.Description, lchProgramGUID, lchLookupGUID ).
            end.    
            lbfObject.Stereotype = "oeLookup".
          end.
        when "Package"
          then do:    
            if available ( lbfXref )
            then do:
              lbfXref.Description = replace( lbfXref.Description, lbfObject.Stereotype, "oeLookupCU" ).
              lbfXref.Description = replace( lbfXref.Description, lchProgramGUID, lchLookupCUGUID ).
            end.    
            lbfObject.Stereotype = "oeLookupCU".
          end.
        otherwise 
          do:
            lobExceptionStack:pushException( 897, 2, "Unexpected element type " + lbfObject.Object_Type + " for " + lbfObject.Name ). 
          end.
      end case.
    end.
  end.
end.

run PostProcessDestruct.
return.
/* ************************************************************************** */         
procedure PostProcessDestruct:
  delete object lobExceptionStack.
  lobExceptionStack = ?.
  empty temp-table ttLookupProgram.
end procedure. 